@@ -5,6 +5,7 @@ module VagrantPlugins
     module Action
       class CreateDomain
         include VagrantPlugins::ProviderLibvirt::Util::ErbTemplate
+        include VagrantPlugins::ProviderLibvirt::Util::StorageUtil
 
         def initialize(app, _env)
           @logger = Log4r::Logger.new('vagrant_libvirt::action::create_domain')
@@ -32,7 +33,8 @@ module VagrantPlugins
           # Gather some info about domain
           @name = env[:domain_name]
           @title = config.title
-          @description = config.description
+          vagrantfile = File.join(env[:root_path], (env[:vagrantfile_name] || "Vagrantfile"))
+          @description = !config.description.empty? ? config.description : "Source: #{vagrantfile}"
           @uuid = config.uuid
           @cpus = config.cpus.to_i
           @cpuset = config.cpuset
@@ -41,6 +43,8 @@ module VagrantPlugins
           @nodeset = config.nodeset
           @features = config.features
           @features_hyperv = config.features_hyperv
+          @clock_offset = config.clock_offset
+          @clock_timers = config.clock_timers
           @shares = config.shares
           @cpu_mode = config.cpu_mode
           @cpu_model = config.cpu_model
@@ -52,11 +56,12 @@ module VagrantPlugins
           @machine_arch = config.machine_arch
           @disk_bus = config.disk_bus
           @disk_device = config.disk_device
+          @disk_driver_opts = config.disk_driver_opts
           @nested = config.nested
           @memory_size = config.memory.to_i * 1024
           @memory_backing = config.memory_backing
           @management_network_mac = config.management_network_mac
-          @domain_volume_cache = config.volume_cache
+          @domain_volume_cache = config.volume_cache || 'default'
           @kernel = config.kernel
           @cmd_line = config.cmd_line
           @emulator_path = config.emulator_path
@@ -80,6 +85,7 @@ module VagrantPlugins
           @tpm_model = config.tpm_model
           @tpm_type = config.tpm_type
           @tpm_path = config.tpm_path
+          @tpm_version = config.tpm_version
 
           # Boot order
           @boot_order = config.boot_order
@@ -87,6 +93,7 @@ module VagrantPlugins
           # Storage
           @storage_pool_name = config.storage_pool_name
           @snapshot_pool_name = config.snapshot_pool_name
+          @domain_volumes = []
           @disks = config.disks
           @cdroms = config.cdroms
 
@@ -136,23 +143,30 @@ module VagrantPlugins
             else
                 pool_name = @storage_pool_name
             end
-            @logger.debug "Search for volume in pool: #{pool_name}"
-            domain_volume = env[:machine].provider.driver.connection.volumes.all(
-              name: "#{@name}.img"
-            ).find { |x| x.pool_name == pool_name }
-            raise Errors::DomainVolumeExists if domain_volume.nil?
-            @domain_volume_path = domain_volume.path
+            @logger.debug "Search for volumes in pool: #{pool_name}"
+            env[:box_volumes].each_index do |index|
+              suffix_index = index > 0 ? "_#{index}" : ''
+              domain_volume = env[:machine].provider.driver.connection.volumes.all(
+                name: "#{@name}#{suffix_index}.img"
+              ).find { |x| x.pool_name == pool_name }
+              raise Errors::DomainVolumeExists if domain_volume.nil?
+              @domain_volumes.push({
+                :dev => (index+1).vdev.to_s,
+                :cache => @domain_volume_cache,
+                :bus => @disk_bus,
+                :path => domain_volume.path,
+                :virtual_size => env[:box_volumes][index][:virtual_size]
+              })
+              end
           end
 
           # If we have a box, take the path from the domain volume and set our storage_prefix.
           # If not, we dump the storage pool xml to get its defined path.
           # the default storage prefix is typically: /var/lib/libvirt/images/
-          if !config.qemu_use_session
-            if env[:machine].config.vm.box
-              storage_prefix = File.dirname(@domain_volume_path) + '/' # steal
-            else
-              storage_prefix = get_disk_storage_prefix(env, @storage_pool_name)
-            end
+          if env[:machine].config.vm.box
+            storage_prefix = File.dirname(@domain_volumes[0][:path]) + '/' # steal
+          else
+            storage_prefix = get_disk_storage_prefix(env, @storage_pool_name)
           end
 
           @disks.each do |disk|
@@ -184,6 +198,8 @@ module VagrantPlugins
                 format_type: disk[:type],
                 path: disk[:absolute_path],
                 capacity: disk[:size],
+                owner: storage_uid(env),
+                group: storage_gid(env),
                 #:allocation => ?,
                 pool_name: disk_pool_name
               )
@@ -222,7 +238,15 @@ module VagrantPlugins
             env[:ui].info(" -- Feature:           #{feature}")
           end
           @features_hyperv.each do |feature|
-            env[:ui].info(" -- Feature (HyperV):  name=#{feature[:name]}, state=#{feature[:state]}")
+            if feature[:name] == 'spinlocks'
+              env[:ui].info(" -- Feature (HyperV):  name=#{feature[:name]}, state=#{feature[:state]}, retries=#{feature[:retries]}")
+            else
+              env[:ui].info(" -- Feature (HyperV):  name=#{feature[:name]}, state=#{feature[:state]}")
+            end
+          end
+          env[:ui].info(" -- Clock offset:      #{@clock_offset}")
+          @clock_timers.each do |timer|
+            env[:ui].info(" -- Clock timer:       #{timer.map { |k,v| "#{k}=#{v}"}.join(', ')}")
           end
           env[:ui].info(" -- Memory:            #{@memory_size / 1024}M")
           unless @nodeset.nil?
@@ -241,8 +265,16 @@ module VagrantPlugins
             env[:ui].info(" -- Base box:          #{env[:machine].box.name}")
           end
           env[:ui].info(" -- Storage pool:      #{@storage_pool_name}")
-          env[:ui].info(" -- Image:             #{@domain_volume_path} (#{env[:box_virtual_size]}G)")
-          env[:ui].info(" -- Volume Cache:      #{@domain_volume_cache}")
+          @domain_volumes.each do |volume|
+            env[:ui].info(" -- Image(#{volume[:device]}):     #{volume[:path]}, #{volume[:virtual_size]}G")
+          end
+
+          if not @disk_driver_opts.empty?
+            env[:ui].info(" -- Disk driver opts:  #{@disk_driver_opts.reject { |k,v| v.nil? }.map { |k,v| "#{k}='#{v}'"}.join(' ')}")
+          else
+            env[:ui].info(" -- Disk driver opts:  cache='#{@domain_volume_cache}'")
+          end
+
           env[:ui].info(" -- Kernel:            #{@kernel}")
           env[:ui].info(" -- Initrd:            #{@initrd}")
           env[:ui].info(" -- Graphics Type:     #{@graphics_type}")
@@ -253,7 +285,13 @@ module VagrantPlugins
           env[:ui].info(" -- Video VRAM:        #{@video_vram}")
           env[:ui].info(" -- Sound Type:	#{@sound_type}")
           env[:ui].info(" -- Keymap:            #{@keymap}")
-          env[:ui].info(" -- TPM Path:          #{@tpm_path}")
+          env[:ui].info(" -- TPM Backend:       #{@tpm_type}")
+          if @tpm_type == 'emulator'
+            env[:ui].info(" -- TPM Model:         #{@tpm_model}")
+            env[:ui].info(" -- TPM Version:       #{@tpm_version}")
+          else
+            env[:ui].info(" -- TPM Path:          #{@tpm_path}")
+          end
 
           @boot_order.each do |device|
             env[:ui].info(" -- Boot device:        #{device}")
