@@ -11,10 +11,8 @@ module VagrantPlugins
         end
 
         def call(env)
-          @env = env
-
           # Get the ports we're forwarding
-          env[:forwarded_ports] = compile_forwarded_ports(env[:machine].config)
+          env[:forwarded_ports] = compile_forwarded_ports(env, env[:machine].config)
 
           # Warn if we're port forwarding to any privileged ports
           env[:forwarded_ports].each do |fp|
@@ -28,51 +26,52 @@ module VagrantPlugins
           # Continue, we need the VM to be booted in order to grab its IP
           @app.call env
 
-          if @env[:forwarded_ports].any?
+          if env[:forwarded_ports].any?
             env[:ui].info I18n.t('vagrant.actions.vm.forward_ports.forwarding')
-            forward_ports
+            forward_ports(env)
           end
         end
 
-        def forward_ports
-          @env[:forwarded_ports].each do |fp|
+        def forward_ports(env)
+          env[:forwarded_ports].each do |fp|
             message_attributes = {
               adapter: fp[:adapter] || 'eth0',
               guest_port: fp[:guest],
               host_port: fp[:host]
             }
 
-            @env[:ui].info(I18n.t(
+            env[:ui].info(I18n.t(
                              'vagrant.actions.vm.forward_ports.forwarding_entry',
-                             message_attributes
+                             **message_attributes
             ))
 
-            if fp[:protocol] == 'udp'
-              @env[:ui].warn I18n.t('vagrant_libvirt.warnings.forwarding_udp')
-              next
-            end
-
             ssh_pid = redirect_port(
-              @env[:machine],
+              env,
+              env[:machine],
               fp[:host_ip] || '*',
               fp[:host],
-              fp[:guest_ip] || @env[:machine].provider.ssh_info[:host],
+              fp[:guest_ip] || env[:machine].provider.ssh_info[:host],
               fp[:guest],
               fp[:gateway_ports] || false
             )
-            store_ssh_pid(fp[:host], ssh_pid)
+            store_ssh_pid(env[:machine], fp[:host], ssh_pid)
           end
         end
 
         private
 
-        def compile_forwarded_ports(config)
+        def compile_forwarded_ports(env, config)
           mappings = {}
 
           config.vm.networks.each do |type, options|
             next if options[:disabled]
 
-            next unless type == :forwarded_port && options[:id] != 'ssh'
+            if options[:protocol] == 'udp'
+              env[:ui].warn I18n.t('vagrant_libvirt.warnings.forwarding_udp')
+              next
+            end
+
+            next if type != :forwarded_port || ( options[:id] == 'ssh' && !env[:machine].provider_config.forward_ssh_port )
             if options.fetch(:host_ip, '').to_s.strip.empty?
               options.delete(:host_ip)
             end
@@ -82,7 +81,7 @@ module VagrantPlugins
           mappings.values
         end
 
-        def redirect_port(machine, host_ip, host_port, guest_ip, guest_port,
+        def redirect_port(env, machine, host_ip, host_port, guest_ip, guest_port,
                           gateway_ports)
           ssh_info = machine.ssh_info
           params = %W(
@@ -107,14 +106,14 @@ module VagrantPlugins
                 "IdentityFile='\"#{pk}\"'"
               end).map { |s| s.prepend('-o ') }.join(' ')
 
-          options += " -o ProxyCommand=\"#{ssh_info[:proxy_command]}\"" if machine.provider_config.connect_via_ssh
+          options += " -o ProxyCommand=\"#{ssh_info[:proxy_command]}\"" if machine.provider_config.proxy_command
 
           # TODO: instead of this, try and lock and get the stdin from spawn...
           ssh_cmd = ''
           if host_port <= 1024
             @@lock.synchronize do
               # TODO: add i18n
-              @env[:ui].info 'Requesting sudo for host port(s) <= 1024'
+              env[:ui].info 'Requesting sudo for host port(s) <= 1024'
               r = system('sudo -v')
               if r
                 ssh_cmd << 'sudo ' # add sudo prefix
@@ -125,14 +124,15 @@ module VagrantPlugins
           ssh_cmd << "ssh -n #{options} #{params}"
 
           @logger.debug "Forwarding port with `#{ssh_cmd}`"
-          log_file = ssh_forward_log_file(host_ip, host_port,
-                                          guest_ip, guest_port)
+          log_file = ssh_forward_log_file(
+            env[:machine], host_ip, host_port, guest_ip, guest_port,
+          )
           @logger.info "Logging to #{log_file}"
           spawn(ssh_cmd, [:out, :err] => [log_file, 'w'], :pgroup => true)
         end
 
-        def ssh_forward_log_file(host_ip, host_port, guest_ip, guest_port)
-          log_dir = @env[:machine].data_dir.join('logs')
+        def ssh_forward_log_file(machine, host_ip, host_port, guest_ip, guest_port)
+          log_dir = machine.data_dir.join('logs')
           log_dir.mkdir unless log_dir.directory?
           File.join(
             log_dir,
@@ -141,8 +141,8 @@ module VagrantPlugins
           )
         end
 
-        def store_ssh_pid(host_port, ssh_pid)
-          data_dir = @env[:machine].data_dir.join('pids')
+        def store_ssh_pid(machine, host_port, ssh_pid)
+          data_dir = machine.data_dir.join('pids')
           data_dir.mkdir unless data_dir.directory?
 
           data_dir.join("ssh_#{host_port}.pid").open('w') do |pid_file|
@@ -169,13 +169,12 @@ module VagrantPlugins
         end
 
         def call(env)
-          @env = env
-
-          if ssh_pids.any?
+          pids = ssh_pids(env[:machine])
+          if pids.any?
             env[:ui].info I18n.t(
               'vagrant.actions.vm.clear_forward_ports.deleting'
             )
-            ssh_pids.each do |tag|
+            pids.each do |tag|
               next unless ssh_pid?(tag[:pid])
               @logger.debug "Killing pid #{tag[:pid]}"
               kill_cmd = ''
@@ -191,7 +190,7 @@ module VagrantPlugins
             end
 
             @logger.info 'Removing ssh pid files'
-            remove_ssh_pids
+            remove_ssh_pids(env[:machine])
           else
             @logger.info 'No ssh pids found'
           end
@@ -201,9 +200,9 @@ module VagrantPlugins
 
         protected
 
-        def ssh_pids
-          glob = @env[:machine].data_dir.join('pids').to_s + '/ssh_*.pid'
-          @ssh_pids = Dir[glob].map do |file|
+        def ssh_pids(machine)
+          glob = machine.data_dir.join('pids').to_s + '/ssh_*.pid'
+          ssh_pids = Dir[glob].map do |file|
             {
               pid: File.read(file).strip.chomp,
               port: File.basename(file)['ssh_'.length..-1 * ('.pid'.length + 1)].to_i
@@ -217,8 +216,8 @@ module VagrantPlugins
           `ps -o command= #{pid}`.strip.chomp =~ /ssh/
         end
 
-        def remove_ssh_pids
-          glob = @env[:machine].data_dir.join('pids').to_s + '/ssh_*.pid'
+        def remove_ssh_pids(machine)
+          glob = machine.data_dir.join('pids').to_s + '/ssh_*.pid'
           Dir[glob].each do |file|
             File.delete file
           end
