@@ -9,9 +9,7 @@ module VagrantPlugins
         end
 
         def call(env)
-          domain = env[:machine].provider.driver.connection.servers.get(env[:machine].id.to_s)
-          raise Errors::NoDomainError if domain.nil?
-          env[:result] = domain.state.to_s == 'running'
+          env[:result] = env[:machine].state.id == :running
 
           @app.call(env)
         end
