@@ -1,3 +1,5 @@
+require 'cgi'
+
 require 'vagrant'
 
 class Numeric
@@ -37,6 +39,11 @@ module VagrantPlugins
       # ID SSH key file
       attr_accessor :id_ssh_key_file
 
+      attr_accessor :proxy_command
+
+      # Forward port with id 'ssh'
+      attr_accessor :forward_ssh_port
+
       # Libvirt storage pool name, where box image and instance snapshots will
       # be stored.
       attr_accessor :storage_pool_name
@@ -59,6 +66,7 @@ module VagrantPlugins
       attr_accessor :management_network_pci_bus
       attr_accessor :management_network_pci_slot
       attr_accessor :management_network_domain
+      attr_accessor :management_network_mtu
 
       # System connection information
       attr_accessor :system_uri
@@ -84,6 +92,8 @@ module VagrantPlugins
       attr_accessor :shares
       attr_accessor :features
       attr_accessor :features_hyperv
+      attr_accessor :clock_offset
+      attr_accessor :clock_timers
       attr_accessor :numa_nodes
       attr_accessor :loader
       attr_accessor :nvram
@@ -93,9 +103,10 @@ module VagrantPlugins
       attr_accessor :machine_virtual_size
       attr_accessor :disk_bus
       attr_accessor :disk_device
+      attr_accessor :disk_driver_opts
       attr_accessor :nic_model_type
       attr_accessor :nested
-      attr_accessor :volume_cache
+      attr_accessor :volume_cache # deprecated, kept for backwards compatibility; use disk_driver
       attr_accessor :kernel
       attr_accessor :cmd_line
       attr_accessor :initrd
@@ -117,6 +128,13 @@ module VagrantPlugins
       attr_accessor :tpm_model
       attr_accessor :tpm_type
       attr_accessor :tpm_path
+      attr_accessor :tpm_version
+
+      # Configure the memballoon
+      attr_accessor :memballoon_enabled
+      attr_accessor :memballoon_model
+      attr_accessor :memballoon_pci_bus
+      attr_accessor :memballoon_pci_slot
 
       # Sets the max number of NICs that can be created
       # Default set to 8. Don't change the default unless you know
@@ -177,10 +195,14 @@ module VagrantPlugins
         @uri               = UNSET_VALUE
         @driver            = UNSET_VALUE
         @host              = UNSET_VALUE
+        @port              = UNSET_VALUE
         @connect_via_ssh   = UNSET_VALUE
         @username          = UNSET_VALUE
         @password          = UNSET_VALUE
         @id_ssh_key_file   = UNSET_VALUE
+        @socket            = UNSET_VALUE
+        @proxy_command     = UNSET_VALUE
+        @forward_ssh_port  = UNSET_VALUE # forward port with id 'ssh'
         @storage_pool_name = UNSET_VALUE
         @snapshot_pool_name = UNSET_VALUE
         @random_hostname   = UNSET_VALUE
@@ -194,6 +216,7 @@ module VagrantPlugins
         @management_network_pci_slot = UNSET_VALUE
         @management_network_pci_bus = UNSET_VALUE
         @management_network_domain = UNSET_VALUE
+        @management_network_mtu = UNSET_VALUE
 
         # System connection information
         @system_uri      = UNSET_VALUE
@@ -215,6 +238,8 @@ module VagrantPlugins
         @shares            = UNSET_VALUE
         @features          = UNSET_VALUE
         @features_hyperv   = UNSET_VALUE
+        @clock_offset      = UNSET_VALUE
+        @clock_timers      = []
         @numa_nodes        = UNSET_VALUE
         @loader            = UNSET_VALUE
         @nvram             = UNSET_VALUE
@@ -223,6 +248,7 @@ module VagrantPlugins
         @machine_virtual_size = UNSET_VALUE
         @disk_bus          = UNSET_VALUE
         @disk_device       = UNSET_VALUE
+        @disk_driver_opts  = {}
         @nic_model_type    = UNSET_VALUE
         @nested            = UNSET_VALUE
         @volume_cache      = UNSET_VALUE
@@ -245,6 +271,12 @@ module VagrantPlugins
         @tpm_model         = UNSET_VALUE
         @tpm_type          = UNSET_VALUE
         @tpm_path          = UNSET_VALUE
+        @tpm_version       = UNSET_VALUE
+
+        @memballoon_enabled = UNSET_VALUE
+        @memballoon_model   = UNSET_VALUE
+        @memballoon_pci_bus = UNSET_VALUE
+        @memballoon_pci_slot = UNSET_VALUE
 
         @nic_adapter_count = UNSET_VALUE
 
@@ -373,10 +405,39 @@ module VagrantPlugins
           raise 'Feature name AND state must be specified'
         end
 
+        if options[:name] == 'spinlocks' && options[:retries].nil?
+          raise 'Feature spinlocks requires retries parameter'
+        end
+
         @features_hyperv = []  if @features_hyperv == UNSET_VALUE
 
-        @features_hyperv.push(name: options[:name],
-                              state: options[:state])
+        if options[:name] == 'spinlocks'
+          @features_hyperv.push(name:   options[:name],
+                             state: options[:state],
+                             retries: options[:retries])
+        else
+          @features_hyperv.push(name:   options[:name],
+                             state: options[:state])
+        end
+      end
+
+      def clock_timer(options = {})
+        if options[:name].nil?
+          raise 'Clock timer name must be specified'
+        end
+
+        options.each do |key, value|
+          case key
+            when :name, :track, :tickpolicy, :frequency, :mode, :present
+              if value.nil?
+                raise "Value of timer option #{key} is nil"
+              end
+            else
+              raise "Unknown clock timer option: #{key}"
+          end
+        end
+
+        @clock_timers.push(options.dup)
       end
 
       def cputopology(options = {})
@@ -551,6 +612,12 @@ module VagrantPlugins
         @smartcard_dev[:source_service] = options[:source_service] if @smartcard_dev[:type] == 'tcp'
       end
 
+      # Disk driver options for primary disk
+      def disk_driver(options = {})
+        supported_opts = [:cache, :io, :copy_on_read, :discard, :detect_zeroes]
+        @disk_driver_opts = options.select { |k,_| supported_opts.include? k }
+      end
+
       # NOTE: this will run twice for each time it's needed- keep it idempotent
       def storage(storage_type, options = {})
         if storage_type == :file
@@ -574,11 +641,13 @@ module VagrantPlugins
         # as will the address unit number (unit=0, unit=1, etc)
 
         options = {
+          type: 'raw',
           bus: 'ide',
           path: nil
         }.merge(options)
 
         cdrom = {
+          type: options[:type],
           dev: options[:dev],
           bus: options[:bus],
           path: options[:path]
@@ -605,6 +674,10 @@ module VagrantPlugins
           allow_existing: options[:allow_existing],
           shareable: options[:shareable],
           serial: options[:serial],
+          io: options[:io],
+          copy_on_read: options[:copy_on_read],
+          discard: options[:discard],
+          detect_zeroes: options[:detect_zeroes],
           pool: options[:pool], # overrides storage_pool setting for additional disks
           wwn: options[:wwn],
         }
@@ -624,15 +697,25 @@ module VagrantPlugins
         @qemu_env.merge!(options)
       end
 
+      def _default_uri
+        # Determine if any settings except driver provided explicitly, if not
+        # and the LIBVIRT_DEFAULT_URI var is set, use that.
+        #
+        # Skipping driver because that may be set on individual boxes rather
+        # than by the user.
+        if [
+            @connect_via_ssh, @host, @username, @password,
+            @id_ssh_key_file, @qemu_use_session, @socket,
+        ].none?{ |v| v != UNSET_VALUE }
+          if ENV.fetch('LIBVIRT_DEFAULT_URI', '') != ""
+            @uri = ENV['LIBVIRT_DEFAULT_URI']
+          end
+        end
+      end
+
       # code to generate URI from from either the LIBVIRT_URI environment
       # variable or a config moved out of the connect action
       def _generate_uri(qemu_use_session)
-
-        # If the LIBVIRT_DEFAULT_URI var is set, we'll use that
-        if ENV.fetch('LIBVIRT_DEFAULT_URI', '') != ""
-          return ENV['LIBVIRT_DEFAULT_URI']
-        end
-
         # builds the Libvirt connection URI from the given driver config
         # Setup connection uri.
         uri = @driver.dup
@@ -652,30 +735,34 @@ module VagrantPlugins
           uri = 'qemu' # use QEMU uri for KVM domain type
         end
 
-        if @connect_via_ssh
-          uri << '+ssh://'
-          uri << @username + '@' if @username
+        # turn on ssh if an ssh key file is explicitly provided
+        if @connect_via_ssh == UNSET_VALUE && @id_ssh_key_file && @id_ssh_key_file != UNSET_VALUE
+          @connect_via_ssh = true
+        end
 
-          uri << ( @host ? @host : 'localhost' )
+        params = {}
+
+        if @connect_via_ssh == true
+          finalize_id_ssh_key_file
+
+          uri << '+ssh://'
+          uri << @username + '@' if @username && @username != UNSET_VALUE
+
+          uri << ( @host && @host != UNSET_VALUE ? @host : 'localhost' )
+
+          params['no_verify'] = '1'
+          params['keyfile'] = @id_ssh_key_file if @id_ssh_key_file
         else
           uri << '://'
-          uri << @host if @host
+          uri << @host if @host && @host != UNSET_VALUE
         end
 
         uri << virt_path
 
-        params = {'no_verify' => '1'}
-
-        if @id_ssh_key_file
-          # set ssh key for access to Libvirt host
-          # if no slash, prepend $HOME/.ssh/
-          @id_ssh_key_file.prepend("#{ENV['HOME']}/.ssh/") if @id_ssh_key_file !~ /\A\//
-          params['keyfile'] = @id_ssh_key_file
-        end
         # set path to Libvirt socket
         params['socket'] = @socket if @socket
 
-        uri << "?" + params.map{|pair| pair.join('=')}.join('&')
+        uri << "?" + params.map{|pair| pair.join('=')}.join('&') if !params.empty?
         uri
       end
 
@@ -688,12 +775,25 @@ module VagrantPlugins
       end
 
       def finalize!
+        _default_uri if @uri == UNSET_VALUE
+
+        # settings which _generate_uri
         @driver = 'kvm' if @driver == UNSET_VALUE
-        @host = nil if @host == UNSET_VALUE
-        @connect_via_ssh = false if @connect_via_ssh == UNSET_VALUE
-        @username = nil if @username == UNSET_VALUE
         @password = nil if @password == UNSET_VALUE
-        @id_ssh_key_file = 'id_rsa' if @id_ssh_key_file == UNSET_VALUE
+        @socket = nil if @socket == UNSET_VALUE
+
+        # If uri isn't set then let's build one from various sources.
+        # Default to passing false for qemu_use_session if it's not set.
+        if @uri == UNSET_VALUE
+          @uri = _generate_uri(@qemu_use_session == UNSET_VALUE ? false : @qemu_use_session)
+        end
+
+        finalize_from_uri
+        finalize_proxy_command
+
+        # forward port with id 'ssh'
+        @forward_ssh_port = false if @forward_ssh_port == UNSET_VALUE
+
         @storage_pool_name = 'default' if @storage_pool_name == UNSET_VALUE
         @snapshot_pool_name = @storage_pool_name if @snapshot_pool_name == UNSET_VALUE
         @storage_pool_path = nil if @storage_pool_path == UNSET_VALUE
@@ -708,24 +808,9 @@ module VagrantPlugins
         @management_network_pci_bus = nil if @management_network_pci_bus == UNSET_VALUE
         @management_network_pci_slot = nil if @management_network_pci_slot == UNSET_VALUE
         @management_network_domain = nil if @management_network_domain == UNSET_VALUE
+        @management_network_mtu = nil if @management_network_mtu == UNSET_VALUE
         @system_uri      = 'qemu:///system' if @system_uri == UNSET_VALUE
 
-        # If uri isn't set then let's build one from various sources.
-        # Default to passing false for qemu_use_session if it's not set.
-        if @uri == UNSET_VALUE
-          @uri = _generate_uri(@qemu_use_session == UNSET_VALUE ? false : @qemu_use_session)
-        end
-
-        # Set qemu_use_session based on the URI if it wasn't set by the user
-        if @qemu_use_session == UNSET_VALUE
-          uri = _parse_uri(@uri)
-          if (uri.scheme.start_with? "qemu") && (uri.path.include? "session")
-            @qemu_use_session = true
-          else
-            @qemu_use_session = false
-          end
-        end
-
         # Domain specific settings.
         @title = '' if @title == UNSET_VALUE
         @description = '' if @description == UNSET_VALUE
@@ -749,6 +834,8 @@ module VagrantPlugins
         @shares = nil if @shares == UNSET_VALUE
         @features = ['acpi','apic','pae'] if @features == UNSET_VALUE
         @features_hyperv = [] if @features_hyperv == UNSET_VALUE
+        @clock_offset = 'utc' if @clock_offset == UNSET_VALUE
+        @clock_timers = [] if @clock_timers == UNSET_VALUE
         @numa_nodes = @numa_nodes == UNSET_VALUE ? nil : _generate_numa
         @loader = nil if @loader == UNSET_VALUE
         @nvram = nil if @nvram == UNSET_VALUE
@@ -757,9 +844,10 @@ module VagrantPlugins
         @machine_virtual_size = nil if @machine_virtual_size == UNSET_VALUE
         @disk_bus = 'virtio' if @disk_bus == UNSET_VALUE
         @disk_device = 'vda' if @disk_device == UNSET_VALUE
+        @disk_driver_opts = {} if @disk_driver_opts == UNSET_VALUE
         @nic_model_type = nil if @nic_model_type == UNSET_VALUE
         @nested = false if @nested == UNSET_VALUE
-        @volume_cache = 'default' if @volume_cache == UNSET_VALUE
+        @volume_cache = nil if @volume_cache == UNSET_VALUE
         @kernel = nil if @kernel == UNSET_VALUE
         @cmd_line = '' if @cmd_line == UNSET_VALUE
         @initrd = '' if @initrd == UNSET_VALUE
@@ -781,6 +869,11 @@ module VagrantPlugins
         @tpm_model = 'tpm-tis' if @tpm_model == UNSET_VALUE
         @tpm_type = 'passthrough' if @tpm_type == UNSET_VALUE
         @tpm_path = nil if @tpm_path == UNSET_VALUE
+        @tpm_version = nil if @tpm_version == UNSET_VALUE
+        @memballoon_enabled = nil if @memballoon_enabled == UNSET_VALUE
+        @memballoon_model = 'virtio' if @memballoon_model == UNSET_VALUE
+        @memballoon_pci_bus = '0x00' if @memballoon_pci_bus == UNSET_VALUE
+        @memballoon_pci_slot = '0x0f' if @memballoon_pci_slot == UNSET_VALUE
         @nic_adapter_count = 8 if @nic_adapter_count == UNSET_VALUE
         @emulator_path = nil if @emulator_path == UNSET_VALUE
 
@@ -872,6 +965,14 @@ module VagrantPlugins
           end
         end
 
+        if !machine.provider_config.volume_cache.nil? and machine.provider_config.volume_cache != UNSET_VALUE
+          machine.ui.warn("Libvirt Provider: volume_cache is deprecated. Use disk_driver :cache => '#{machine.provider_config.volume_cache}' instead.")
+
+          if !machine.provider_config.disk_driver_opts.empty?
+            machine.ui.warn("Libvirt Provider: volume_cache has no effect when disk_driver is defined.")
+          end
+        end
+
         { 'Libvirt Provider' => errors }
       end
 
@@ -885,11 +986,103 @@ module VagrantPlugins
           c += other.cdroms
           result.cdroms = c
 
+          result.disk_driver_opts = disk_driver_opts.merge(other.disk_driver_opts)
+          
+          c = clock_timers.dup
+          c += other.clock_timers
+          result.clock_timers = c
+
           c = qemu_env != UNSET_VALUE ? qemu_env.dup : {}
           c.merge!(other.qemu_env) if other.qemu_env != UNSET_VALUE
           result.qemu_env = c
         end
       end
+
+      private
+
+      def finalize_from_uri
+        # Parse uri to extract individual components
+        uri = _parse_uri(@uri)
+
+        # only set @connect_via_ssh if not explicitly to avoid overriding
+        # and allow an error to occur if the @uri and @connect_via_ssh disagree
+        @connect_via_ssh = uri.scheme.include? "ssh" if @connect_via_ssh == UNSET_VALUE
+
+        # Set qemu_use_session based on the URI if it wasn't set by the user
+        if @qemu_use_session == UNSET_VALUE
+          if (uri.scheme.start_with? "qemu") && (uri.path.include? "session")
+            @qemu_use_session = true
+          else
+            @qemu_use_session = false
+          end
+        end
+
+        # Extract host and username values from uri if provided, otherwise nil
+        @host = uri.host
+        @port = uri.port
+        @username = uri.user
+        if uri.query
+          params = CGI.parse(uri.query)
+          @id_ssh_key_file = params['keyfile'].first if params.has_key?('keyfile')
+        end
+
+        finalize_id_ssh_key_file
+      end
+
+      def resolve_ssh_key_file(key_file)
+        # set ssh key for access to Libvirt host
+        # if no slash, prepend $HOME/.ssh/
+        key_file.prepend("#{ENV['HOME']}/.ssh/") if key_file && key_file !~ /\A\//
+
+        key_file
+      end
+
+      def finalize_id_ssh_key_file
+        # resolve based on the following roles
+        #  1) if @connect_via_ssh is set to true, and id_ssh_key_file not current set,
+        #     set default if the file exists
+        #  2) if supplied the key name, attempt to expand based on user home
+        #  3) otherwise set to nil
+
+        if @connect_via_ssh == true && @id_ssh_key_file == UNSET_VALUE
+          # set default if using ssh while allowing a user using nil to disable this
+          id_ssh_key_file = resolve_ssh_key_file('id_rsa')
+          id_ssh_key_file = nil if !File.file?(id_ssh_key_file)
+        elsif @id_ssh_key_file != UNSET_VALUE
+          id_ssh_key_file = resolve_ssh_key_file(@id_ssh_key_file)
+        else
+          id_ssh_key_file = nil
+        end
+
+        @id_ssh_key_file = id_ssh_key_file
+      end
+
+      def finalize_proxy_command
+        if @connect_via_ssh
+          if @proxy_command == UNSET_VALUE
+            proxy_command = "ssh '#{@host}' "
+            proxy_command << "-p #{@port} " if @port
+            proxy_command << "-l '#{@username}' " if @username
+            proxy_command << "-i '#{@id_ssh_key_file}' " if @id_ssh_key_file
+            proxy_command << '-W %h:%p'
+          else
+            inputs = { host: @host }
+            inputs << { port: @port } if @port
+            inputs[:username] = @username if @username
+            inputs[:id_ssh_key_file] = @id_ssh_key_file if @id_ssh_key_file
+
+            proxy_command = @proxy_command
+            # avoid needing to escape '%' symbols
+            inputs.each do |key, value|
+              proxy_command.gsub!("{#{key}}", value)
+            end
+          end
+
+          @proxy_command = proxy_command
+        else
+          @proxy_command = nil
+        end
+      end
     end
   end
 end
